﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Serialization;
using System.Windows.Forms;

namespace TeklaEPMOpenAPIExample
{
    public partial class TeklaEPMOpenAPIExampleF : Form
    {
        FabSuite.FabSuiteAPI.IFabSuiteAPI fsAPI = new FabSuite.FabSuiteAPI.FabSuiteAPI();

        string connection_ipAddress = "localhost";  // ip address of the Tekla EPM Database
        string connection_port = "3306";  // default port is 3306
        string connection_username = "admin"; // a Tekla EPM user account and password 
        string connection_password = "fab";

        public TeklaEPMOpenAPIExampleF()
        {
            InitializeComponent();
        }

        private void ConnectButton_Click(object sender, EventArgs e)
        {
            //declare the request object
            reqFabSuiteAPI.fsreqFabSuiteXMLRequest request = new reqFabSuiteAPI.fsreqFabSuiteXMLRequest();

            //declare and populate the Connect command object
            reqFabSuiteAPI.fsreqConnect connectCommand = new reqFabSuiteAPI.fsreqConnect();
            connectCommand.IPAddress = connection_ipAddress;
            connectCommand.PortNumber = connection_port;
            connectCommand.Username = connection_username;
            connectCommand.Password = connection_password;

            //set the Connect command object in the request object
            request.Connect = new reqFabSuiteAPI.fsreqConnect[1];
            request.Connect[0] = connectCommand;

            //execute the command and get the response object
            resFabSuiteAPI.fsresFabSuiteXMLResponse response = executeCommand(request);

            //first check for an XMLError element
            if (response.XMLError != null && response.XMLError.Length > 0)
                MessageBox.Show(response.XMLError[0]);  //fatal error
            else
            {
                //The get the Connect response object and check if it was successful
                resFabSuiteAPI.fsresConnect connectResponse = response.Connect[0];
                if (!connectResponse.Successful)
                    MessageBox.Show(connectResponse.ErrorMessage);  //command failed
                else
                    MessageBox.Show("connected!");
            }
        }

        private void ReadProductionControlJobsButton_Click(object sender, EventArgs e)
        {
            ProductionControlJobListBox.Items.Clear();

            //declare the request object
            reqFabSuiteAPI.fsreqFabSuiteXMLRequest request = new reqFabSuiteAPI.fsreqFabSuiteXMLRequest();

            //declare the GetProductionControlJobs command object
            reqFabSuiteAPI.fsreqGetProductionControlJobs getProductionControlJobsCommand = new reqFabSuiteAPI.fsreqGetProductionControlJobs();

            //set the GetProductionControlJobs command object in the request object
            request.GetProductionControlJobs = new reqFabSuiteAPI.fsreqGetProductionControlJobs[1];
            request.GetProductionControlJobs[0] = getProductionControlJobsCommand;

            //execute the command and get the response object
            resFabSuiteAPI.fsresFabSuiteXMLResponse response = executeCommand(request);

            //first check for an XMLError element
            if (response.XMLError != null && response.XMLError.Length > 0)
                MessageBox.Show(response.XMLError[0]);  //fatal error
            else
            {
                //The get the GetProductionControlJobs response object and check if it was successful
                resFabSuiteAPI.fsresGetProductionControlJobs getProductionControlJobsResponse = response.GetProductionControlJobs[0];
                if (!getProductionControlJobsResponse.Successful)
                    MessageBox.Show(getProductionControlJobsResponse.ErrorMessage);  //command failed
                else
                {
                    // command successful, show each job in the list box.
                    for (int i = 0; i < getProductionControlJobsResponse.ProductionControlJob.Length; i++)
                    {
                        resFabSuiteAPI.fsresGetProductionControlJobs_ProductionControlJob productionControlJob = getProductionControlJobsResponse.ProductionControlJob[ i ];
                        ProductionControlJobListBox.Items.Add(productionControlJob.JobNumber);
                    }
                }
            }
        }

        private void DisconnectButton_Click(object sender, EventArgs e)
        {
            //declare the request object
            reqFabSuiteAPI.fsreqFabSuiteXMLRequest request = new reqFabSuiteAPI.fsreqFabSuiteXMLRequest();

            //declare and populate the CloseRemote command object
            reqFabSuiteAPI.fsreqClose command = new reqFabSuiteAPI.fsreqClose();

            request.Close = new reqFabSuiteAPI.fsreqClose[1];
            request.Close[0] = command;

            //execute the command and get the response object
            resFabSuiteAPI.fsresFabSuiteXMLResponse response = executeCommand(request);

            //first check for an XMLError element
            if (response.XMLError != null && response.XMLError.Length > 0)
                MessageBox.Show(response.XMLError[0]);  //fatal error
            else
            {
                //The get the CloseRemote response object and check if it was successful
                resFabSuiteAPI.fsresClose closeResponse = response.Close[0];
                if (!closeResponse.Successful)
                    MessageBox.Show(closeResponse.ErrorMessage);  //command failed
                else
                    MessageBox.Show("disconnected successfully");
            }
        }

        resFabSuiteAPI.fsresFabSuiteXMLResponse executeCommand(reqFabSuiteAPI.fsreqFabSuiteXMLRequest request)
        {
            //set the xml writer settings
            XmlWriterSettings xmlWriterSettings = new XmlWriterSettings();
            xmlWriterSettings.Indent = true;
            xmlWriterSettings.IndentChars = "  ";
            xmlWriterSettings.NewLineHandling = NewLineHandling.Entitize;

            //serialize the request object to a memory stream
            XmlSerializer requestSerializer = new XmlSerializer(typeof(reqFabSuiteAPI.fsreqFabSuiteXMLRequest));
            MemoryStream requestMemoryStream = new MemoryStream();
            XmlWriter xmlWriter = XmlWriter.Create(requestMemoryStream, xmlWriterSettings);
            requestSerializer.Serialize(xmlWriter, request);

            //turn the memory stream into a byte array and get a utf-8 string of the request xml
            string requestXML = System.Text.Encoding.UTF8.GetString(requestMemoryStream.ToArray());

            //execute the request against the API and get the response xml string
            string responseXML = fsAPI.FabSuiteXML(requestXML);

            //create the XML reader object
            XmlTextReader xmlReader = new NamespaceOverrideXmlTextReader(new StringReader(responseXML));

            //deserialize the XML into the response object and return
            XmlSerializer responseSerializer = new XmlSerializer(typeof(resFabSuiteAPI.fsresFabSuiteXMLResponse));
            return (resFabSuiteAPI.fsresFabSuiteXMLResponse)responseSerializer.Deserialize(xmlReader);
        }
    }

    //Future versions of Tekla EPM will implement new API functionality and will have a different schema namespace.
    //The Tekla EPM API will always be backwards-compatible but the different namespace (eg. ...v0105.xsd) will make the serialization fail.
    //The NamespaceOverrideXmlTextReader class overrides the namespace that is actually in the XML response so that it will succeed even with a different scema namespace.
    public class NamespaceOverrideXmlTextReader : XmlTextReader
    {
        public NamespaceOverrideXmlTextReader(StringReader stringReader) : base(stringReader) { }

        public override string NamespaceURI
        {
            get { return "http://www.fabsuite.com/xml/fabsuite-xml-response-v0108.xsd"; }
        }
    }
}
